//==============================================================================
// BapSupport.c - Other support functions that need a home...
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 		Robert V Mahoney Jr
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================

#include "BapLib.h"

#define _BA_I2C_EEPROM_					0xA0	// 400kHz internal I2CEEPROM 4K byte 24LC32

#define _EEPROM_WAIT_TIMEOUT_			10


#ifdef __cplusplus
extern "C" {
#endif

unsigned long 	lTimerCnt;   //used in WTimer overflow. Will keep a 16 bit overflow so we have a 32 bit timer 

//--------------------------------------------------------------------
//[InitSystemTimer] - Initialize the system timer
//--------------------------------------------------------------------
void InitSystemTimer(void)
{
#ifdef BASICATOMPRO28
	TA.TMA.BYTE = 0; 					//clock / 8192					; Low resolution clock - used for timeouts...
	IRR1.BIT.IRRTA = 0;					// clear out the overflow flag
#else
	TB1.TMB1.BYTE = 0; 					//clock / 8192					; Low resolution clock - used for timeouts...
	IRR2.BIT.IRRTB1 = 0;				// clear out the overflow flag
#endif
	lTimerCnt = 0 ;
	DBGPrintf("Before Enable TimerA\r");

#ifdef BASICATOMPRO28
	enable (TIMERAINT);	 		//enable timer interrupt
#else
	enable (TIMERB1INT);	 		//enable timer interrupt
#endif
}

//--------------------------------------------------------------------
//[GetCurrentTime] - Gets the Timer value from our overflow counter as well as the TCA counter.  It
//                makes sure of consistancy. That is it is very posible that 
//                after we grabed the timers value it overflows, before we grab the other part
//                so we check to make sure it is correct and if necesary regrab things.
//--------------------------------------------------------------------
	
unsigned long GetCurrentTime(void)
{
	unsigned long lCurrentTime;
#ifdef BASICATOMPRO28
	lCurrentTime = lTimerCnt + TA.TCA;
	
	// handle wrap
	if (lTimerCnt != (lCurrentTime & 0xffffff00))
		lCurrentTime = lTimerCnt + TA.TCA;
#else
	lCurrentTime = (lTimerCnt & 0xffffff00l) | TB1.TCB1;
	
	// handle wrap
	if (lTimerCnt != (lCurrentTime & 0xffffff00l))
		lCurrentTime = (lTimerCnt & 0xffffff00) | TB1.TCB1;
#endif
	return lCurrentTime;
}

unsigned long micros(void)
{
	return (GetCurrentTime() * WTIMERTICSPERMSMUL) * (1000/ WTIMERTICSPERMSDIV);   // currently div is 125 so 
}

unsigned long millis(void)
{
	return (GetCurrentTime() * WTIMERTICSPERMSMUL) / WTIMERTICSPERMSDIV;   // currently div is 125 so 
}

long map(long x, long in_min, long in_max, long out_min, long out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}


//--------------------------------------------------------------------
//[FTimeHasElapsed] - Gets the Timer value from our overflow counter as well as the TCA counter.  It
//                makes sure of consistancy. That is it is very posible that 
//                after we grabed the timers value it overflows, before we grab the other part
//                so we check to make sure it is correct and if necesary regrab things.
//--------------------------------------------------------------------
bool FTimeHasElapsed(u32 lTimeSave, u32 lTimeInMS)
{
	u32 lTimer = GetCurrentTime();

    if (lTimer >= lTimeSave)	// normal case timer has not wrapped
      	return (((lTimer-lTimeSave)*WTIMERTICSPERMSMUL) / WTIMERTICSPERMSDIV) >= lTimeInMS;

	// Timer wrapped	
   	return (((0xffffffffL - lTimer + lTimeSave + 1)*WTIMERTICSPERMSMUL) / WTIMERTICSPERMSDIV) > lTimeInMS;
	
}


#ifdef BASICATOMPRO28
extern "C" void TIMERAV() __attribute__ ((interrupt_handler));
extern "C" void TIMERAV()
{
	lTimerCnt += 256;		// increment our counter TCA is the low order byte so increment past it...
	IRR1.BIT.IRRTA = 0;		// clear out the overflow flag
}
#else
extern "C" void TIMERB1() __attribute__ ((interrupt_handler));
extern "C" void TIMERB1()
{
	lTimerCnt += 256;		// increment our counter TCA is the low order byte so increment past it...
	IRR2.BIT.IRRTB1 = 0;					// clear out the overflow flag
}
#endif


long iabs(long s)
{
	if (s >= 0)
		return s;
	return -s;
}

long imin (long a, long b)
{
	if (a <= b)
		return a;
	return b;	
}

long imax (long a, long b)
{
	if (a >= b)
		return a;
	return b;	
}


//--------------------------------------------------------------------
// [enable] - Enable specific interrupts or master switch
//--------------------------------------------------------------------
void enable(INTS ints)
{
	switch (ints){
		case _MASTER_INT_:
			and_ccr(0x7f);					// should clear the interrupt disable mask
			break;

#ifdef BASICATOMPRO28
		case TIMERAINT:
			IENR1.BIT.IENTA = 1; 		// ENABLE TIMERAINT 
			break;
#else
		case TIMERB1INT:
			IENR2.BIT.IENTB1 = 1; 		// ENABLE TIMERB1INT 
			break;
#endif			
	}
}

//--------------------------------------------------------------------
// [disable] - Disable specific interrupts - or master switch
//--------------------------------------------------------------------
void disable(INTS ints)
{
	switch (ints)
	{
		case _MASTER_INT_:
			or_ccr(0x80);				// This should turn on the Int flag which will mask interrupts)
			break;

#ifdef BASICATOMPRO28
		case TIMERAINT:
			IENR1.BIT.IENTA = 0; 		// DISABLE TIMERAINT 
			break;
#else
		case TIMERB1INT:
			IENR2.BIT.IENTB1 = 0; 		// DISABLE TIMERB1INT 
			break;
#endif			
	}
}


//==============================================================================
// [DebugMemoryDump] - striped down version of rprintfMemoryDump
//==============================================================================
//#ifdef DEBUG
void DebugMemoryDump(const byte* data, int off, int len)
{
    int x;
    int c;
    int line;
    const byte * b = data;

    for(line = 0; line < ((len % 16 != 0) ? (len / 16) + 1 : (len / 16)); line++)  {
    	int line16 = line * 16;
    	DBGPrintf("%4x| ", (unsigned int)data + line16);

        // put hex values
        for(x = 0; x < 16; x++) {
            if(x + line16 < len) {
                c = b[off + x + line16];
				DBGPrintf("%2x ", c);
            }
            else
				DBGPrintf("   ");
        }
		DBGPrintf("| ");

        // put ascii values
        for(x = 0; x < 16; x++) {
            if(x + line16 < len) {
                c = b[off + x + line16];
                DBGPrintf("%c",((c > 0x1f) && (c < 0x7f))? c : '.');
             }
             else
             	DBGPrintf(" ");
        }
        DBGPrintf("\n\r");
    }
}

//#endif



__inline__ u32 get_sp(void) __attribute__ ((always_inline));
__inline__ u32 get_sp(void)
{
      u32 sp_val;
      asm("mov.l er7, %0":"=g"(sp_val));
      return(sp_val);
}


// read from the EEPROM
u8 Read (u16 _wAddress, u16 _wReadCount, u8 *_pbData)
{
	return SI2CRead (_SDAI_, _SCLI_, _BA_I2C_EEPROM_, _I2C_ADDRTYPE_16BIT_, _wAddress, _pbData, _wReadCount);
}

// write to the EEPROM
u8 Write (u16 _wAddress, u16 _wWriteCount, u8 *_pbData)
{
	u8 bError;
	u8 bData;
	u16 wElapsed = 0;

	bError = SI2CWrite (_SDAI_, _SCLI_, _BA_I2C_EEPROM_, _I2C_ADDRTYPE_16BIT_, _wAddress, _pbData, _wWriteCount);
	if (bError == _STATUS_NO_ERROR_) {
		// wait for ack from EEPROM to indicate write operation is complete
		for (;;) {
			bError = SI2CRead (_SDAI_, _SCLI_, _BA_I2C_EEPROM_, _I2C_ADDRTYPE_16BIT_, _wAddress, &bData, 1);
			if (bError == _STATUS_NO_ERROR_)
				break;
			wElapsed++;
			if (wElapsed > _EEPROM_WAIT_TIMEOUT_)
				break;
		};
	}
	return bError;
}




#ifdef __cplusplus
}
#endif
